'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));

// Idea taken from JSS / MaterialUI createGenerateClassName
// XXX: this is relying that the import order between Client and Server side JS is the same
//      i don't know if this will break during code splitting

var keyCounter = 0;
var prod = process.env.NODE_ENV === 'production';

var createGenerateComponentKey = function createGenerateComponentKey() {
  var counter = keyCounter++;
  return function (Component, props) {
    if (prod) return 'c-' + counter;
    return Component.name + '-' + counter;
  };
};

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/** High Order Component which
 * @param {(props: Object, bag: Object) => Promise} getData -
 * @param {(data: Object, ownProps: Object) => Object} mapDataToProps -
 * @param {(props: Object, nextProps: Object) => bool} shouldGetData - (optional)
 * @param {(branch: Object, extra?: Object) => Object} mapArgsToProps - (optional)
 * @param {(component: ReactComponent, props: Object) => String} generateComponentKey - (optional)
 * @return {ReactComponent}
 */
var withGetInitialData = function withGetInitialData(_ref) {
  var _getData = _ref.getData,
      mapDataToProps = _ref.mapDataToProps,
      _ref$shouldGetData = _ref.shouldGetData,
      shouldGetData = _ref$shouldGetData === undefined ? function () {
    return false;
  } : _ref$shouldGetData,
      _ref$mapArgsToProps = _ref.mapArgsToProps,
      mapArgsToProps = _ref$mapArgsToProps === undefined ? function () {
    return {};
  } : _ref$mapArgsToProps,
      _ref$generateComponen = _ref.generateComponentKey,
      generateComponentKey = _ref$generateComponen === undefined ? createGenerateComponentKey() : _ref$generateComponen;
  return function (Component) {
    var GetInitialData = function (_React$Component) {
      _inherits(GetInitialData, _React$Component);

      _createClass(GetInitialData, null, [{
        key: 'getInitialData',

        /** Static method called during SSR to fetch data
         * @param {Object} [branch] - Extra data
         * @param {Object} [extra] - Extra data
         * @return {Object} - { key: ComponentKey, promise: Promise that the data will be fetched }
         */
        value: function getInitialData(branch, extra) {
          var promise = Promise.resolve(null);
          var props = mapArgsToProps(branch, extra);
          var key = generateComponentKey(Component, props);

          if (_getData) {
            var data = null;
            promise = _getData(props, {
              // Ignore setLoading as the server will render after
              setLoading: function setLoading() {},
              // Use setData in order to retrieve data
              setData: function setData(d) {
                return data = d;
              }
            }).then(function () {
              return data;
            });
          }

          return {
            promise: promise,
            key: key
          };
        }
      }]);

      function GetInitialData(props) {
        _classCallCheck(this, GetInitialData);

        var _this = _possibleConstructorReturn(this, (GetInitialData.__proto__ || Object.getPrototypeOf(GetInitialData)).call(this, props));

        _this.key = generateComponentKey(Component, props);
        _this.state = {
          data: props.getInitialData(_this.key),
          // We are faking the value of isLoading in order to prevent a rendering.
          // If the Component hasn't loaded, the user will call `setLoading(true)` on `getData`
          // If user doesn't do it, the value will be overriden in the `setData`.
          // There is also the case, that the user doesn't call `setLoading` / `setData`, the `isLoading` value
          // will be incorrectly setted, but also, the user won't use it. So it doesn't really matter.
          isLoading: !props.hasLoadedComponent(_this.key)
        };
        return _this;
      }

      /** React `componentDidMount` phase.
       * As far as data fetching, it is only called on the client side when the component is mounted.
       * A double fetch has to be prevented (fetch data when the server has already provided it)
       *
       * Check if data has to be fetched, if it does, it calls `getData` of Component.
       * The idea of this method is to prevent the double fetch when hydrating SSR code.
       * It checks if the Component's key has been fetched.
       */


      _createClass(GetInitialData, [{
        key: 'componentDidMount',
        value: function componentDidMount() {
          // If Component didn't implement, do nothing
          if (!_getData) return;
          if (this.props.hasLoadedComponent(this.key)) return;
          this.getData();
        }

        /** React `componentWillReceiveProps` phase.
         * It is called when props have changed.
         * We'll have to check if the change involves a new data fetch.
         */

      }, {
        key: 'componentWillReceiveProps',
        value: function componentWillReceiveProps(nextProps) {
          // If Component didn't implement, do nothing
          if (!_getData) return;
          if (!shouldGetData(this.props, nextProps)) return;

          this.dismissLoadedComponent();
          this.getData(nextProps);
        }
      }, {
        key: 'componentWillUnmount',
        value: function componentWillUnmount() {
          this.dismissLoadedComponent();
        }
      }, {
        key: 'render',
        value: function render() {
          var _state = this.state,
              isLoading = _state.isLoading,
              data = _state.data;

          var props = Object.assign({}, this.props, mapDataToProps(data || {}, this.props));

          return React.createElement(Component, _extends({ isLoading: isLoading }, props));
        }

        /** Performs data getting
         */

      }, {
        key: 'getData',
        value: function getData(nextProps) {
          var _this2 = this;

          // If Component didn't implement, do nothing
          if (!_getData) return;
          _getData(nextProps || this.props, {
            setLoading: function setLoading(b) {
              return b !== _this2.state.isLoading && _this2.setState({ isLoading: b });
            },
            setData: function setData(d) {
              return _this2.setState({ isLoading: false, data: d });
            }
          });
        }

        /**
         */

      }, {
        key: 'dismissLoadedComponent',
        value: function dismissLoadedComponent() {
          // If Component didn't implement, do nothing
          if (!_getData) return;
          this.props.dismissLoadedComponent(this.key);
        }
      }]);

      return GetInitialData;
    }(React.Component);

    GetInitialData.propTypes = {
      getInitialData: PropTypes.func.isRequired,
      hasLoadedComponent: PropTypes.func.isRequired,
      dismissLoadedComponent: PropTypes.func.isRequired
    };

    return GetInitialData;
  };
};

exports.withGetInitialData = withGetInitialData;
//# sourceMappingURL=react-data-ssr.js.map
